@@ -228,8 +228,23 @@ data Switch
     | SwStrm TxStreamData
 
 sender :: Connection -> IO ()
-sender conn = handleLogT logAction $ forever $ sendP conn
+sender conn = handleLogT logAction loop
   where
+    loop = do
+        exit <- atomically $ do
+            done <- readTVar $ connDone conn
+            a <- isEmptyPingSTM (connLDCC conn)
+            b <- isEmptyOutputSTM conn
+            c <- isEmptyStreamSTM conn
+            if done
+                then return (a && b && c)
+                else if (not a || not b || not c) then return False else retry
+        if exit
+            then
+                E.throwIO ExitConnection
+            else do
+                sendP conn
+                loop
     logAction msg = connDebugLog conn ("debug: sender: " <> msg)
 
 sendP :: Connection -> IO ()
