@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -6,6 +7,7 @@ module TLSSpec where
 import Data.Bits
 import qualified Data.ByteString as BS
 import Data.Maybe
+import Network.TLS.Extra.Cipher
 import Test.Hspec
 
 import Network.QUIC.Internal
@@ -22,9 +24,12 @@ instance Eq (ServerTrafficSecret a) where
 
 spec :: Spec
 spec = do
+    ----------------------------------------------------------------
+    -- RFC 9001
+    --
     describe "test vector for version 1" $ do
         let ver = Version1
-        it "describes the examples of Keys" $ do
+        it "describes the examples of Keys (RFC 9001: A.1)" $ do
             ----------------------------------------------------------------
             -- shared keys
             let dcID = makeCID (dec16s "8394c8f03e515708")
@@ -49,7 +54,7 @@ spec = do
             let shp = headerProtectionKey ver defaultCipher (Secret sis)
             shp `shouldBe` Key (dec16 "c206b8d9b9f0f37644430b490eeaa314")
 
-        it "describes the examples of Client Initial" $ do
+        it "describes the examples of Client Initial (RFC 9001: A.2)" $ do
             let dcID = makeCID (dec16s "8394c8f03e515708")
                 ClientTrafficSecret cis = clientInitialSecret ver dcID
                 ckey = aeadKey ver defaultCipher (Secret cis)
@@ -91,7 +96,7 @@ spec = do
             let Mask mask = protectionMask defaultCipher chp sample
             BS.take 5 mask `shouldBe` dec16 "437b9aec36"
 
-        it "describes the examples of Server Initial" $ do
+        it "describes the examples of Server Initial (RFC 9001: A.3)" $ do
             let dcID = makeCID (dec16s "8394c8f03e515708")
                 ServerTrafficSecret sis = serverInitialSecret ver dcID
                 skey = aeadKey ver defaultCipher (Secret sis)
@@ -132,7 +137,7 @@ spec = do
             let Mask mask = protectionMask defaultCipher shp sample
             BS.take 5 mask `shouldBe` dec16 "2ec0d8356a"
 
-        it "describes the examples of Retry" $ do
+        it "describes the examples of Retry (RFC 9001: A.4)" $ do
             let wire0 =
                     dec16 "ff000000010008f067a5502a4262b5746f6b656e04a265ba2eff4d829058fb3f0f2496ba"
             (ipkt, rest) <- decodePacket wire0 True
@@ -146,7 +151,41 @@ spec = do
                     r0 `shouldBe` r1
                 _ -> error "Retry version 1"
 
-    describe "test vector for version 2" $ do
+{- FOURMOLU_DISABLE -}
+#ifndef USE_FUSION
+        it
+            "describes the examples of ChaCha20-Poly1305 Short Header Packet (RFC 9001: A.5)"
+            $ do
+                let cipher = cipher13_CHACHA20_POLY1305_SHA256
+                    secret =
+                        Secret $
+                            dec16 "9ac312a7f877468ebe69422748ad00a15443f18203a07d6060f688f30f21632b"
+                    key = aeadKey ver cipher secret
+                    iv = initialVector ver cipher secret
+                    hp = headerProtectionKey ver cipher secret
+                    ku = nextSecret ver cipher secret
+                    payloadCipherText = niteEncrypt cipher key iv (dec16 "01") (AssDat $ dec16 "4200bff4") 654360564
+                    sample = Sample $ dec16 "5e5cd55c41f69080575d7999c25a5bfb"
+                    mask = protectionMask cipher hp sample
+                key
+                    `shouldBe` Key (dec16 "c6d98ff3441c3fe1b2182094f69caa2ed4b716b65488960a7a984979fb23e1c8")
+                iv
+                    `shouldBe` IV (dec16 "e0459b3474bdd0e44a41c144")
+                hp
+                    `shouldBe` Key (dec16 "25a282b9e82f06f21f488917a4fc8f1b73573685608597d0efcb076b0ab7a7a4")
+                ku
+                    `shouldBe` Secret
+                        (dec16 "1223504755036d556342ee9361d253421a826c9ecdf3c7148684b36b714881f9")
+                payloadCipherText
+                    `shouldBe` Just (dec16 "65", dec16 "5e5cd55c41f69080575d7999c25a5bfb")
+                mask `shouldBe` Mask (dec16 "aefefe7d03")
+#endif
+{- FOURMOLU_ENSABLE -}
+
+    ----------------------------------------------------------------
+    -- RFC 9369
+    --
+    describe "test vector for version 2 (RFC 9369: A1)" $ do
         let ver = Version2
         it "describes the examples of Keys" $ do
             ----------------------------------------------------------------
@@ -173,7 +212,7 @@ spec = do
             let shp = headerProtectionKey ver defaultCipher (Secret sis)
             shp `shouldBe` Key (dec16 "edf6d05c83121201b436e16877593c3a")
 
-        it "describes the examples of Client Initial" $ do
+        it "describes the examples of Client Initial (RFC 9369: A2)" $ do
             let dcID = makeCID (dec16s "8394c8f03e515708")
                 ClientTrafficSecret cis = clientInitialSecret ver dcID
                 ckey = aeadKey ver defaultCipher (Secret cis)
@@ -215,7 +254,7 @@ spec = do
             let Mask mask = protectionMask defaultCipher chp sample
             BS.take 5 mask `shouldBe` dec16 "94a0c95e80"
 
-        it "describes the examples of Server Initial" $ do
+        it "describes the examples of Server Initial (RFC 9369: A3)" $ do
             let dcID = makeCID (dec16s "8394c8f03e515708")
                 ServerTrafficSecret sis = serverInitialSecret ver dcID
                 skey = aeadKey ver defaultCipher (Secret sis)
@@ -256,7 +295,7 @@ spec = do
             let Mask mask = protectionMask defaultCipher shp sample
             BS.take 5 mask `shouldBe` dec16 "4dd92e91ea"
 
-        it "describes the examples of Retry" $ do
+        it "describes the examples of Retry (RFC 9369: A4)" $ do
             let wire0 =
                     dec16 "cf6b3343cf0008f067a5502a4262b5746f6b656ec8646ce8bfe33952d955543665dcc7b6"
             (ipkt, rest) <- decodePacket wire0 True
@@ -270,6 +309,37 @@ spec = do
                     r0 `shouldBe` r1
                 _ -> error "Retry version 2"
 
+{- FOURMOLU_DISABLE -}
+#ifndef USE_FUSION
+        it
+            "describes the examples of ChaCha20-Poly1305 Short Header Packet (RFC 9369: A.5)"
+            $ do
+                let cipher = cipher13_CHACHA20_POLY1305_SHA256
+                    secret =
+                        Secret $
+                            dec16 "9ac312a7f877468ebe69422748ad00a15443f18203a07d6060f688f30f21632b"
+                    key = aeadKey ver cipher secret
+                    iv = initialVector ver cipher secret
+                    hp = headerProtectionKey ver cipher secret
+                    ku = nextSecret ver cipher secret
+                    payloadCipherText = niteEncrypt cipher key iv (dec16 "01") (AssDat $ dec16 "4200bff4") 654360564
+                    sample = Sample $ dec16 "e7b6b932bc27d786f4bc2bb20f2162ba"
+                    mask = protectionMask cipher hp sample
+                key
+                    `shouldBe` Key (dec16 "3bfcddd72bcf02541d7fa0dd1f5f9eeea817e09a6963a0e6c7df0f9a1bab90f2")
+                iv
+                    `shouldBe` IV (dec16 "a6b5bc6ab7dafce30ffff5dd")
+                hp
+                    `shouldBe` Key (dec16 "d659760d2ba434a226fd37b35c69e2da8211d10c4f12538787d65645d5d1b8e2")
+                ku
+                    `shouldBe` Secret
+                        (dec16 "c69374c49e3d2a9466fa689e49d476db5d0dfbc87d32ceeaa6343fd0ae4c7d88")
+                payloadCipherText
+                    `shouldBe` Just (dec16 "0a", dec16 "e7b6b932bc27d786f4bc2bb20f2162ba")
+                mask `shouldBe` Mask (dec16 "97580e32bf")
+#endif
+{- FOURMOLU_ENABLE -}
+
 serverCRYPTOframe :: BS.ByteString
 serverCRYPTOframe =
     dec16 $
